import com.boondocks.taxcontrols.TaxControl;
import com.boondocks.taxcontrols.Taxes;
import com.boondocks.taxcontrols.Tax;
import com.boondocks.taxcontrols.TaxControlException;
import java.util.*;
import javax.swing.*;
import java.text.*;

/** 
 * Demo dialog for the TaxControls control.
 * A simple application demonstrating common usage.
 * For instructional use only.
 * @author Caliber Technology LLC
 * @version 1.21
 * @since 1.4
 * @see http://www.taxupdate.com
 */
public class Demo extends javax.swing.JFrame {
    
    /** Creates new form Demo */
    public Demo(String path) {
        initComponents();
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setLocationRelativeTo(null);        // center of screen

        // Initial setup
        m_tax = new TaxControl();
        
        // find the data file
        m_tax.setDataFilename(path);
        
        // go ahead and load the data.
        // This is the only time we want to do this, since 
        // it takes a while.
        try{
            // This step is not strictly needed.
            // Data loading happens automatically with 
            // methods requiring it (getTaxes, setSelectedTax, etc.)
            // 'True' here would mean load in the 'background'
            // (on a separate thread).
            m_tax.refresh(false);
            buildLists();
        }
        catch(TaxControlException e){
            e.printStackTrace();
        }
    }
    /**
     * Set up the lists of taxes by type:
     */
    void buildLists() throws TaxControlException {
        // for sorting:
        m_state = new TreeSet(Collator.getInstance());
        m_local = new TreeSet(Collator.getInstance());
        
        m_state.add("(None)");
        m_local.add("(None)");
        
        Taxes taxes = m_tax.getTaxes();
        Enumeration en = taxes.elements();
        while (en.hasMoreElements()){
            Tax tax = (Tax)en.nextElement();

            // There are 5 types of 'Tax' objects: Federal ("F"),
            // State ("S"), Local ("L"), Other ("O"), and 
            // Private ("P" - used internally).
            String sType = tax.getTaxType();
            if (sType.equals("S"))
                m_state.add(tax.getName());
            else if (sType.equals("L"))
                m_local.add(tax.getName());
        }
        cmbState.setModel(new DefaultComboBoxModel(m_state.toArray()));
        cmbLocal.setModel(new DefaultComboBoxModel(m_local.toArray()));
    }
    /** This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        pnlGeneral = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        cmbPayPeriod = new javax.swing.JComboBox();
        cmbState = new javax.swing.JComboBox();
        cmbLocal = new javax.swing.JComboBox();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        txtEarnings = new javax.swing.JTextField();
        jLabel9 = new javax.swing.JLabel();
        txtExemptions = new javax.swing.JTextField();
        jLabel10 = new javax.swing.JLabel();
        cmbFilingStatus = new javax.swing.JComboBox();
        jLabel11 = new javax.swing.JLabel();
        txtPayDate = new javax.swing.JTextField();
        pnlTaxes = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        txtTaxFederal = new javax.swing.JTextField();
        jLabel4 = new javax.swing.JLabel();
        txtTaxState = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        txtTaxLocal = new javax.swing.JTextField();
        jLabel8 = new javax.swing.JLabel();
        txtTaxFICA = new javax.swing.JTextField();
        cmdCompute = new javax.swing.JButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setTitle("TaxControls Demo");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        pnlGeneral.setLayout(new java.awt.GridBagLayout());

        pnlGeneral.setBorder(new javax.swing.border.TitledBorder(null, "Options", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 1, 12), new java.awt.Color(0, 0, 255)));
        jLabel2.setText("Pay Period:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlGeneral.add(jLabel2, gridBagConstraints);

        cmbPayPeriod.setFont(new java.awt.Font("Dialog", 0, 11));
        cmbPayPeriod.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Week", "2 Weeks", "2 Per Month", "Month", "Quarter", "Year" }));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        pnlGeneral.add(cmbPayPeriod, gridBagConstraints);

        cmbState.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Initial line for sizing" }));
        cmbState.setToolTipText("For completeness, this list contains states which have no withholding tax.");
        cmbState.setPreferredSize(new java.awt.Dimension(200, 24));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlGeneral.add(cmbState, gridBagConstraints);

        cmbLocal.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Initial line for sizing" }));
        cmbLocal.setPreferredSize(new java.awt.Dimension(200, 25));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlGeneral.add(cmbLocal, gridBagConstraints);

        jLabel5.setText("State:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlGeneral.add(jLabel5, gridBagConstraints);

        jLabel6.setText("Local:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlGeneral.add(jLabel6, gridBagConstraints);

        jLabel7.setText("Earnings:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlGeneral.add(jLabel7, gridBagConstraints);

        txtEarnings.setFont(new java.awt.Font("Dialog", 0, 11));
        txtEarnings.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        txtEarnings.setText("1000.00");
        txtEarnings.setPreferredSize(new java.awt.Dimension(120, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlGeneral.add(txtEarnings, gridBagConstraints);

        jLabel9.setText("Exemptions:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlGeneral.add(jLabel9, gridBagConstraints);

        txtExemptions.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        txtExemptions.setText("1");
        txtExemptions.setPreferredSize(new java.awt.Dimension(85, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        pnlGeneral.add(txtExemptions, gridBagConstraints);

        jLabel10.setText("Filing Status:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlGeneral.add(jLabel10, gridBagConstraints);

        cmbFilingStatus.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Single", "Head of Household", "Qualified Widow(er)", "Married", "Married Filing Separately", "Married Both Employed" }));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlGeneral.add(cmbFilingStatus, gridBagConstraints);

        jLabel11.setText("Pay Date:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlGeneral.add(jLabel11, gridBagConstraints);

        txtPayDate.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        Date today = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("M/d/yyyy");
        String sDate = sdf.format(today);
        txtPayDate.setText(sDate);
        txtPayDate.setPreferredSize(new java.awt.Dimension(85, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        pnlGeneral.add(txtPayDate, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        getContentPane().add(pnlGeneral, gridBagConstraints);

        pnlTaxes.setLayout(new java.awt.GridBagLayout());

        pnlTaxes.setBorder(new javax.swing.border.TitledBorder(null, "Taxes", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 1, 12), new java.awt.Color(0, 0, 255)));
        jLabel3.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        jLabel3.setText("Federal:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 6);
        pnlTaxes.add(jLabel3, gridBagConstraints);

        txtTaxFederal.setEditable(false);
        txtTaxFederal.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        txtTaxFederal.setMinimumSize(new java.awt.Dimension(60, 20));
        txtTaxFederal.setPreferredSize(new java.awt.Dimension(100, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlTaxes.add(txtTaxFederal, gridBagConstraints);

        jLabel4.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        jLabel4.setText("State:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 6);
        pnlTaxes.add(jLabel4, gridBagConstraints);

        txtTaxState.setEditable(false);
        txtTaxState.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        txtTaxState.setPreferredSize(new java.awt.Dimension(100, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlTaxes.add(txtTaxState, gridBagConstraints);

        jLabel1.setText("Local:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlTaxes.add(jLabel1, gridBagConstraints);

        txtTaxLocal.setEditable(false);
        txtTaxLocal.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        txtTaxLocal.setPreferredSize(new java.awt.Dimension(100, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlTaxes.add(txtTaxLocal, gridBagConstraints);

        jLabel8.setText("FICA:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlTaxes.add(jLabel8, gridBagConstraints);

        txtTaxFICA.setEditable(false);
        txtTaxFICA.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        txtTaxFICA.setPreferredSize(new java.awt.Dimension(120, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 5;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlTaxes.add(txtTaxFICA, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        getContentPane().add(pnlTaxes, gridBagConstraints);

        cmdCompute.setText("Compute");
        cmdCompute.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmdComputeActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        getContentPane().add(cmdCompute, gridBagConstraints);

        pack();
    }//GEN-END:initComponents

    private void cmdComputeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmdComputeActionPerformed
        try{
            compute();
        }
        catch (TaxControlException e){
            e.printStackTrace();
        }
    }//GEN-LAST:event_cmdComputeActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    /**
     * Compute the taxes and update the form.
     */
    void compute() throws TaxControlException{
        // pay periods per year:
        String period = (String)cmbPayPeriod.getSelectedItem();
        if (period.equalsIgnoreCase("week"))
            m_tax.setPayPeriodsPerYear(52);
        else if (period.equalsIgnoreCase("2 weeks"))
            m_tax.setPayPeriodsPerYear(26);
        else if (period.equalsIgnoreCase("2 per month"))
            m_tax.setPayPeriodsPerYear(24);
        else if (period.equalsIgnoreCase("month"))
            m_tax.setPayPeriodsPerYear(12);
        else if (period.equalsIgnoreCase("quarter"))
            m_tax.setPayPeriodsPerYear(4);
        else if (period.equalsIgnoreCase("year"))
            m_tax.setPayPeriodsPerYear(1);
        else
            m_tax.setPayPeriodsPerYear(12);
        
        // state tax name:
        String statename = (String)cmbState.getSelectedItem();
        
        // local tax name:
        String localname = (String)cmbLocal.getSelectedItem();
        
        // paydate:
        String sPayDate = txtPayDate.getText();
        SimpleDateFormat sdf = new SimpleDateFormat("M/d/yyyy");
        Date dPay = new Date();
        try{
            dPay = sdf.parse(sPayDate);
        }
        catch (ParseException e){}
            
        m_tax.setPayDate(dPay);
        
        // earnings:
        String sEarn = txtEarnings.getText();
        NumberFormat f = NumberFormat.getNumberInstance();
        float fEarn = 0.0f;
        try{
            fEarn = f.parse(sEarn).floatValue();
        }
        catch (ParseException e){}
            
        m_tax.setEarnings(fEarn);
        String sExem= txtExemptions.getText();
        f = NumberFormat.getNumberInstance();
        int nExem = 0;
        try{
            nExem = f.parse(sExem).intValue();
        }
        catch (ParseException e){}
            
        // get the Filing Status:
        int nFiling = cmbFilingStatus.getSelectedIndex();
        m_tax.setFilingStatus(nFiling);
        
        // exemptions:
        m_tax.setExemptions(nExem);
        m_tax.setStateExemptions(nExem);
        
        // for formatting the output:
        NumberFormat cf = NumberFormat.getCurrencyInstance();
        
        // First federal ("FIT"):
        m_tax.setSelectedTax("FIT");
        float amt = m_tax.getTaxAmount();
        txtTaxFederal.setText(cf.format(amt));
        
        // Now FICA:
        m_tax.setSelectedTax("FICA");
        amt = m_tax.getTaxAmount();
        txtTaxFICA.setText(cf.format(amt));
        
        // The state tax:
        try{
            // if the selection is "(None)", we'll get an exception:
            m_tax.setSelectedTax(statename);
            amt = m_tax.getTaxAmount();
            txtTaxState.setText(cf.format(amt));
        }
        catch (TaxControlException e){
            txtTaxState.setText(cf.format(0));
        }
        
        // The local tax:
        try{
            // if the selection is "(None)", we'll get an exception:
            m_tax.setSelectedTax(localname);
            amt = m_tax.getTaxAmount();
            txtTaxLocal.setText(cf.format(amt));
        }
        catch (TaxControlException e){
            txtTaxLocal.setText(cf.format(0));
        }
    }
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        if (args.length==0){
            System.out.println("Usage: Demo path\n\nwhere path is location of tax file.\n");
            System.exit(1);
        }
        String path = args[0];
        new Demo(path).setVisible(true);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox cmbFilingStatus;
    private javax.swing.JComboBox cmbLocal;
    private javax.swing.JComboBox cmbPayPeriod;
    private javax.swing.JComboBox cmbState;
    private javax.swing.JButton cmdCompute;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel pnlGeneral;
    private javax.swing.JPanel pnlTaxes;
    private javax.swing.JTextField txtEarnings;
    private javax.swing.JTextField txtExemptions;
    private javax.swing.JTextField txtPayDate;
    private javax.swing.JTextField txtTaxFICA;
    private javax.swing.JTextField txtTaxFederal;
    private javax.swing.JTextField txtTaxLocal;
    private javax.swing.JTextField txtTaxState;
    private static final long serialVersionUID = 7229567526422776147L;
    // End of variables declaration//GEN-END:variables

    /**
     * The TaxControls component
     */
    TaxControl m_tax;
    TreeSet m_state, m_local;
}
